@@ -1356,9 +1356,8 @@ test_that("arg check fuzz tests for fold()", {
   # Testing side effects
   expect_error(
     xpectr::strip_msg(fold_2(data = df, k = 3, cat_col = "diagnosis", num_col = NULL, id_col = "score", method = "n_dist", id_aggregation_fn = sum, extreme_pairing_levels = 1, num_fold_cols = 1, unique_fold_cols_only = TRUE, max_iters = 5, handle_existing_fold_cols = "keep_warn", parallel = FALSE)),
-    xpectr::strip(paste0("2 assertions failed:\n * Variable 'data[[id_col]]': Must be",
-                         " of type 'factor', not 'double'.\n * The value in 'data[[cat",
-                         "_col]]' must be constant within each ID.")),
+    xpectr::strip(paste0("1 assertions failed:\n * Variable 'data[[id_col]]': Must be",
+                         " of type 'factor', not 'double'.")),
     fixed = TRUE)
 
   # Testing fold_2(data = df, k = 3, cat_col = "diagnosis"...
@@ -1967,3 +1966,80 @@ test_that("arg check fuzz tests for fold()", {
 
 
 })
+
+
+test_that("fold() works with group_by()", {
+  xpectr::set_test_seed(42)
+
+  df <- data.frame(
+    "n" = c(1, 2, 3, 4, 2, 1, 5, 2, 1, 9),
+    "s" = c(4, 4, 4, 4, 7, 7, 7, 7, 1, 1),
+    "c" = as.character(c(4, 4, 6, 6, 7, 7, 7, 8, 8, 1)),
+    "f" = as.factor(as.character(c(4, 4, 6, 6, 7, 7, 7, 8, 8, 1))),
+    stringsAsFactors = FALSE
+  )
+
+
+  ## Testing 'xpectr::suppress_mw( df %>% dplyr::group_by(...'              ####
+  ## Initially generated by xpectr
+  xpectr::set_test_seed(42)
+  # Assigning output
+  output_19148 <- xpectr::suppress_mw(
+      df %>%
+        dplyr::group_by(s) %>%
+        fold(k=2)
+    )
+  # Testing class
+  expect_equal(
+    class(output_19148),
+    c("grouped_df", "tbl_df", "tbl", "data.frame"),
+    fixed = TRUE)
+  # Testing column values
+  expect_equal(
+    output_19148[["n"]],
+    c(1, 9, 1, 2, 3, 4, 2, 1, 5, 2),
+    tolerance = 1e-4)
+  expect_equal(
+    output_19148[["s"]],
+    c(1, 1, 4, 4, 4, 4, 7, 7, 7, 7),
+    tolerance = 1e-4)
+  expect_equal(
+    output_19148[["c"]],
+    c("8", "1", "4", "4", "6", "6", "7", "7", "7", "8"),
+    fixed = TRUE)
+  expect_equal(
+    output_19148[["f"]],
+    structure(c(5L, 1L, 2L, 2L, 3L, 3L, 4L, 4L, 4L, 5L), .Label = c("1",
+      "4", "6", "7", "8"), class = "factor"))
+  expect_equal(
+    output_19148[[".folds"]],
+    structure(c(2L, 1L, 1L, 2L, 2L, 1L, 2L, 1L, 1L, 2L), .Label = c("1",
+      "2"), class = "factor"))
+  # Testing column names
+  expect_equal(
+    names(output_19148),
+    c("n", "s", "c", "f", ".folds"),
+    fixed = TRUE)
+  # Testing column classes
+  expect_equal(
+    xpectr::element_classes(output_19148),
+    c("numeric", "numeric", "character", "factor", "factor"),
+    fixed = TRUE)
+  # Testing column types
+  expect_equal(
+    xpectr::element_types(output_19148),
+    c("double", "double", "character", "integer", "integer"),
+    fixed = TRUE)
+  # Testing dimensions
+  expect_equal(
+    dim(output_19148),
+    c(10L, 5L))
+  # Testing group keys
+  expect_equal(
+    colnames(dplyr::group_keys(output_19148)),
+    ".folds",
+    fixed = TRUE)
+  ## Finished testing 'xpectr::suppress_mw( df %>% dplyr::group_by(...'     ####
+
+
+})
