@@ -838,9 +838,8 @@ test_that("input checks fuzz testing of partition()", {
   # Testing side effects
   expect_error(
     xpectr::strip_msg(partition_2(data = df, p = 0.5, cat_col = "diagnosis", num_col = NULL, id_col = "score", id_aggregation_fn = sum, extreme_pairing_levels = 1, force_equal = FALSE, list_out = FALSE)),
-    xpectr::strip(paste0("2 assertions failed:\n * Variable 'data[[id_col]]': Must be",
-                         " of type 'factor', not 'double'.\n * The value in 'data[[cat",
-                         "_col]]' must be constant within each ID.")),
+    xpectr::strip(paste0("1 assertions failed:\n * Variable 'data[[id_col]]': Must be",
+                         " of type 'factor', not 'double'.")),
     fixed = TRUE)
 
   # Testing partition_2(data = df, p = 0.5, cat_col = "dia...
@@ -1213,3 +1212,82 @@ test_that("input checks fuzz testing of partition()", {
   ## Finished testing 'partition_2'                                           ####
   #
 })
+
+
+test_that("partition() works with group_by()", {
+  xpectr::set_test_seed(1)
+
+  df <- data.frame(
+    "participant" = factor(rep(c("1", "2", "3", "4", "5", "6"), 3)),
+    "age" = rep(c(25, 65, 34), 3),
+    "diagnosis" = factor(rep(c("a", "b", "a", "a", "b", "b"), 3)),
+    "score" = c(34, 23, 54, 23, 56, 76, 43, 56, 76, 42, 54, 1, 5, 76, 34, 76, 23, 65)
+  )
+
+  ## Testing 'xpectr::suppress_mw( df %>% dplyr::group_by(...'              ####
+  ## Initially generated by xpectr
+  xpectr::set_test_seed(42)
+  # Assigning output
+  output_12655 <- xpectr::suppress_mw(
+      df %>%
+        dplyr::group_by(diagnosis) %>%
+        partition(p=0.8, id_col = "participant")
+    )
+  # Testing class
+  expect_equal(
+    class(output_12655),
+    "data.frame",
+    fixed = TRUE)
+  # Testing column values
+  expect_equal(
+    output_12655[["participant"]],
+    structure(c(1L, 1L, 1L, 3L, 3L, 3L, 4L, 4L, 4L, 2L, 2L, 2L, 5L,
+      5L, 5L, 6L, 6L, 6L), .Label = c("1", "2", "3", "4", "5", "6"),
+      class = "factor"))
+  expect_equal(
+    output_12655[["age"]],
+    c(25, 25, 25, 34, 34, 34, 25, 25, 25, 65, 65, 65, 65, 65, 65, 34,
+      34, 34),
+    tolerance = 1e-4)
+  expect_equal(
+    output_12655[["diagnosis"]],
+    structure(c(1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 2L, 2L, 2L, 2L,
+      2L, 2L, 2L, 2L, 2L), .Label = c("a", "b"), class = "factor"))
+  expect_equal(
+    output_12655[["score"]],
+    c(34, 5, 43, 54, 76, 34, 42, 23, 76, 23, 76, 56, 56, 54, 23, 1,
+      76, 65),
+    tolerance = 1e-4)
+  expect_equal(
+    output_12655[[".partitions"]],
+    structure(c(2L, 2L, 2L, 1L, 1L, 1L, 1L, 1L, 1L, 2L, 2L, 2L, 1L,
+      1L, 1L, 1L, 1L, 1L), .Label = c("1", "2"), class = "factor"))
+  # Testing column names
+  expect_equal(
+    names(output_12655),
+    c("participant", "age", "diagnosis", "score", ".partitions"),
+    fixed = TRUE)
+  # Testing column classes
+  expect_equal(
+    xpectr::element_classes(output_12655),
+    c("factor", "numeric", "factor", "numeric", "factor"),
+    fixed = TRUE)
+  # Testing column types
+  expect_equal(
+    xpectr::element_types(output_12655),
+    c("integer", "double", "integer", "double", "integer"),
+    fixed = TRUE)
+  # Testing dimensions
+  expect_equal(
+    dim(output_12655),
+    c(18L, 5L))
+  # Testing group keys
+  expect_equal(
+    colnames(dplyr::group_keys(output_12655)),
+    character(0),
+    fixed = TRUE)
+  ## Finished testing 'xpectr::suppress_mw( df %>% dplyr::group_by(...'     ####
+
+
+})
+
